require 'active_support/concern'

module Gemthief
  class Executive
    module WindowManagement
      extend ActiveSupport::Concern

      included do
        include Curses
      end

      def init_curses!
        init_screen
        cbreak
        noecho
        srand
        curs_set 0
        trap("INT") { exit 0; }
      end

      def refresh_curses!
        stdscr.noutrefresh
        active_scene.view.refresh!
        doupdate
      end

      def root_window
        stdscr
      end

      class Panel
        def initialize(screen_x, screen_y, screen_width, screen_height, content_width, content_height, content_x=0, content_y=0)
          @screen_x       = screen_x
          @screen_y       = screen_y
          @screen_width   = screen_width
          @screen_height  = screen_height
          @content_width  = content_width
          @content_height = content_height
          @content_x      = content_x
          @content_y      = content_y

          @outer = Curses::Pad.new(@content_height, @content_width)
          @inner = @outer.subpad(@content_height, @content_width,1,1)
        end

        # TODO: Use Forwardable for these and other methods
        def clear
          @inner.clear
        end
        def setpos(at_line, at_col)
          @inner.setpos(at_line, at_col)
        end
        def addstr(char)
          @inner.addstr("@")
        end

        def noutrefresh
          @outer.noutrefresh(
            @content_y, @content_x, 
            @screen_y, @screen_x, 
            screen_maxrow, screen_maxcol
          )
          @outer.box(?|, ?-)
          @inner.noutrefresh(
            @content_y, @content_x,
            @screen_y+1, @screen_x+1,
            screen_maxrow-2, screen_maxcol-2
          )
        end

        def dimensions
          { x: @width, y: @height }
        end

        def screen_maxrow
          @screen_y + @screen_height
        end

        def screen_maxcol
          @screen_x + @screen_width
        end
      end
    end
  end
end
